package main

import (
	"crypto/hmac"
	"crypto/sha256"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"html/template"
	"io/ioutil"
	"net/http"
	"net/url"
	"time"
)

// 验证公钥
const CAPTCHA_ID string = "d4b6324f31f5584958fefb4081c1a3bf"

// 验证私钥
const CAPTCHA_KEY string = "8d8846f4f1033bf6529e6e456981afea"

// 验证服务地址
const API_SERVER string = "https://captcha.alicaptcha.com"

// 二次校验接口
const URL = API_SERVER + "/validate" + "?captcha_id=" + CAPTCHA_ID

//  index.html
func index(response http.ResponseWriter, req *http.Request) {
	html_response, err := template.ParseFiles("static/index.html")
	if err == nil {
		html_response.Execute(response, "")
	}
}

// login
func login(writer http.ResponseWriter, request *http.Request) {
	if request.Method != "GET" {
		writer.WriteHeader(405)
		return
	}
	// 前端传回的数据
	datas := request.URL.Query()
	lot_number := datas["lot_number"][0]
	captcha_output := datas["captcha_output"][0]
	pass_token := datas["pass_token"][0]
	gen_time := datas["gen_time"][0]
	// 生成签名
	// 生成签名使用标准的hmac算法，使用用户当前完成验证的流水号lot_number作为原始消息message，使用客户验证私钥作为key
	// 采用sha256散列算法将message和key进行单向散列生成最终的 “sign_token” 签名
	sign_token := hmac_encode(CAPTCHA_KEY, lot_number)

	// 向验证服务转发前端数据 + “sign_token” 签名
	form_data := make(url.Values)
	form_data["lot_number"] = []string{lot_number}
	form_data["captcha_output"] = []string{captcha_output}
	form_data["pass_token"] = []string{pass_token}
	form_data["gen_time"] = []string{gen_time}
	form_data["sign_token"] = []string{sign_token}

	// 发起post请求
	// 设置5s超时
	cli := http.Client{Timeout: time.Second * 5}
	resp, err := cli.PostForm(URL, form_data)
	if err != nil || resp.StatusCode != 200 {
		// 当请求发生异常时，应放行通过，以免阻塞业务
		fmt.Println("服务接口异常: ")
		fmt.Println(err)
		writer.Write([]byte("success"))
		return
	}

	res_json, _ := ioutil.ReadAll(resp.Body)
	var res_map map[string]interface{}
	// 根据验证服务返回的用户验证状态, 网站主进行自己的业务逻辑
	// 响应json数据如：{"result": "success", "reason": "", "captcha_args": {}}

	if err = json.Unmarshal(res_json, &res_map); err != nil {
		fmt.Println("Json数据解析错误")
		writer.Write([]byte("success"))
		return
	}

	result := res_map["result"]
	if result == "success" {
		fmt.Println("验证通过")
		writer.Write([]byte("success"))
	} else {
		reason := res_map["reason"]
		fmt.Print("验证失败: ")
		fmt.Print(reason)
		writer.Write([]byte("fail"))
	}

}

// hmac-sha256 加密：  CAPTCHA_KEY,lot_number
func hmac_encode(key string, data string) string {
	mac := hmac.New(sha256.New, []byte(key))
	mac.Write([]byte(data))
	return hex.EncodeToString(mac.Sum(nil))
}

func main() {
	http.HandleFunc("/", index)
	http.HandleFunc("/login", login)
	http.ListenAndServe(":8082", nil)
}
